package com.qmth.themis.business.forkjoin;

import com.google.gson.Gson;
import com.qmth.themis.business.dto.ExamStudentDtoImport;
import com.qmth.themis.business.entity.TBTaskHistory;
import com.qmth.themis.business.entity.TEExamStudent;
import com.qmth.themis.business.enums.TaskTypeExecEnum;
import com.qmth.themis.business.service.TBTaskHistoryService;
import com.qmth.themis.business.service.TEExamStudentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.RecursiveTask;

@Component
public class ForkJoinTask extends RecursiveTask<List<TEExamStudent>> {
    private final static Logger log = LoggerFactory.getLogger(ForkJoinTask.class);

    private int threshold = 500, start, end;//阀值，开始，结束，类型
    private List<Object> examStudentDtoImportList;
    private TBTaskHistory tbTaskHistory = null;

    public ForkJoinTask() {

    }

    public ForkJoinTask(int threshold, int start, int end, List<Object> examStudentDtoImportList, TBTaskHistory tbTaskHistory) {
        this.threshold = threshold;
        this.start = start;
        this.end = end;
        this.examStudentDtoImportList = examStudentDtoImportList;
        this.tbTaskHistory = tbTaskHistory;
    }

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TBTaskHistoryService tbTaskHistoryService;

    @Override
    protected List<TEExamStudent> compute() {
        Gson gson = new Gson();
        List<TEExamStudent> teExamStudentList = new ArrayList<>();
        if ((end - start) <= threshold) {
            for (int i = start; i <= end; i++) {
                ExamStudentDtoImport examStudentDtoImport = (ExamStudentDtoImport) examStudentDtoImportList.get(i);
                TEExamStudent teExamStudent = gson.fromJson(gson.toJson(examStudentDtoImport), TEExamStudent.class);
                teExamStudentList.add(teExamStudent);
            }
            BigDecimal bigDecimal = new BigDecimal(100);
            int size = examStudentDtoImportList.size();
            BigDecimal endBigDecimal = new BigDecimal(end);
            BigDecimal progress = new BigDecimal(Double.valueOf(endBigDecimal.divide(new BigDecimal(size), 2, BigDecimal.ROUND_HALF_UP).multiply(bigDecimal).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue())).setScale(0, BigDecimal.ROUND_HALF_UP);
            teExamStudentService.saveBatch(teExamStudentList);
            if (progress.intValue() == 100) {
                tbTaskHistory.setStatus(TaskTypeExecEnum.finish.ordinal());
                tbTaskHistory.setSummary("共处理了" + size + "条数据");
                tbTaskHistory.setFinishTime(new Date());
            } else {
                tbTaskHistory.setStatus(TaskTypeExecEnum.running.ordinal());
                tbTaskHistory.setSummary("正在处理第" + start + "条至" + end + "条数据");
            }
            tbTaskHistory.setProgress(progress.doubleValue());
            tbTaskHistoryService.updateById(tbTaskHistory);
            log.info("start:{},end:{}", start, end);
        } else {
            // 将大任务分解成两个小任务
            int middle = (start + end) / 2;
            ForkJoinTask left = new ForkJoinTask(threshold, start, middle, examStudentDtoImportList, tbTaskHistory);
            ForkJoinTask right = new ForkJoinTask(threshold, middle + 1, end, examStudentDtoImportList, tbTaskHistory);
            invokeAll(left, right);
            teExamStudentList.addAll(left.join());
            teExamStudentList.addAll(right.join());
        }
        return teExamStudentList;
    }
}
